#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
	// Parte 2 - Cdigo do Exemplo
	ofLogNotice("[PAPER DETECTOR]") << " Configure CV ";
	contourFinder.setMinAreaRadius(10); // configura menor rea .
	contourFinder.setMaxAreaRadius(200); // configura maior rea
	thresholdCV = 100; // inicializa o nvel da camera.
	//contourFinder.setTargetColor(targetColor, trackingColorMode); // configura cor / padro
	contourFinder.setThreshold(thresholdCV); // configura nvel

	ofLogNotice("[PAPER DETECTOR]") << " Configure Camera ";
	camWidth = 640; camHeight = 480;
	cam.initGrabber(camWidth,camHeight);
	cam.listDevices();
	unwarped.allocate(camWidth, camHeight, OF_IMAGE_COLOR); // 	// ALLOCATE RAW IMAGE
	warpedPaper.allocate(camWidth,camHeight,OF_IMAGE_COLOR);
}

//--------------------------------------------------------------
void ofApp::update(){
	cam.update();
	float area;

	if(cam.isFrameNew()){
		//ofLogNotice("[PAPER DETECTOR]") << "New Frame";
		vector<cv::Point> maxQuad;								//maior quadrado na imagem = folha de papel.
		float maxArea = -numeric_limits<float>::infinity();		//escolhe pela maior rea.

		contourFinder.findContours(cam); // realiza a procura por folhas brancas em um fundo preto

		int n = contourFinder.size(); 
		for(int i = 0; i < n; i++){
			//ofLogNotice("[PAPER DETECTOR]") << "update: " + ofToString(i);
			ofPolyline minAreRect = toOf(contourFinder.getMinAreaRect(i));
			vector<cv::Point> quad = contourFinder.getFitQuad(i);

			area = contourFinder.getContourArea(i);   //encontrar o maior quadrado na imagem = folha de papel.
			if (area > maxArea) {
				//ofLogNotice("[PAPER DETECTOR]") << "1. " + ofToString(area) + " - " + ofToString(maxArea); 
				maxArea = area;
				maxQuad = quad;
			}
		}

		// teste se foi encontrado algo parecido com um retngulo. 
		this->foundInfra = false;
		if ( this->findForm(maxQuad) == "RECTANGLE"){
			this->foundInfra = true;

			// warp paper
			vector<Point2f> warpPoints;
			warpPoints.push_back(maxQuad[3]); // para organizar a ordem dos pontos.
			warpPoints.push_back(maxQuad[0]);
			warpPoints.push_back(maxQuad[1]);
			warpPoints.push_back(maxQuad[2]);

			//copy(maxQuad.begin(), maxQuad.end(), back_inserter(warpPoints));
			unwarpPerspective(cam, unwarped, warpPoints);
			unwarped.update();
		}
		//ofLogNotice("[PAPER DETECTOR]") << "2. " + ofToString(area) + " - " + ofToString(maxArea) + " - " + ofToString(this->foundInfra);
	}
}

//--------------------------------------------------------------
void ofApp::draw(){
	ofSetColor(ofColor::white);
	cam.draw(0,0);

	ofNoFill();
	int n = contourFinder.size();
	for(int i = 0; i < n; i++) {
		// smallest rectangle that fits the contour
		ofSetColor(ofColor::red);
		ofPolyline minAreRect = toOf(contourFinder.getMinAreaRect(i));
		minAreRect.draw();

		// convex hull of the contour
		ofSetColor(yellowPrint);
		ofPolyline convexHull = toOf(contourFinder.getConvexHull(i));
		convexHull.draw();

		ofSetColor(cyanPrint);
		vector<cv::Point> quad = contourFinder.getFitQuad(i);
		vector<cv::Point>::iterator it;
		for (it = quad.begin(); it != quad.end(); it++){
			ofCircle(toOf(*it),3);
		}
	}

	ofSetColor(255);
	unwarped.draw(640, 0);

	if(this->foundInfra){ ofDrawBitmapStringHighlight("Found infra",10,ofGetHeight() - 80);	}
	ofDrawBitmapStringHighlight(ofToString(this->thresholdCV),10,ofGetHeight() - 100);
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){


	// Parte 2 - Cdigo do Exemplo
	switch (key) {
	case 's': case 'S':
		cam.videoSettings();
		break;
	default:
		break;
	};

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}


static double angle(ofPoint pt1, ofPoint pt2, ofPoint pt0)
{
	double dx1 = pt1.x - pt0.x;
	double dy1 = pt1.y - pt0.y;
	double dx2 = pt2.x - pt0.x;
	double dy2 = pt2.y - pt0.y;
	return (dx1*dx2 + dy1*dy2)/sqrt((dx1*dx1 + dy1*dy1)*(dx2*dx2 + dy2*dy2) + 1e-10);
}

// pegadinha para achar formas geomtricas - to do: circle and square
// forma de identificao: numero de vrtices e coseno entre angulos.
// http://opencv-code.com/tutorials/detecting-simple-shapes-in-an-image/
string ofApp::findForm(const ofPolyline &poly){

	vector<ofPoint> corners = poly.getVertices();

	if ( corners.size() == 3) return "TRIANGLE";
	if ( corners.size() >= 4 && corners.size() <= 6){
		int vtc = corners.size();
		vector<double> cos;
		for( int j = 2; j < vtc+1; j++)	
			cos.push_back(angle(corners[j%vtc], corners[j-2], corners[j-1]));
		sort(cos.begin(),cos.end());

		double minCos = cos.front();
		double maxCos = cos.back();

		if( vtc == 4 && minCos >= -0.1 && maxCos <= 0.3) {//return "RECTANGLE";
			 cv::Rect r = cv::boundingRect(toCv(poly));
            double ratio = std::abs(1 - (double)r.width / r.height);
			return (ratio <= 0.02) ? "SQUARE" : "RECTANGLE";

		}
		if( vtc == 5 && minCos >= -0.34 && maxCos <= -0.27) return "PENTAGON";
		if( vtc == 6 && minCos >= -0.55 && maxCos <= 0.45) 	return "HEXAGON";
	} else {
		// testar se circulo
	}

	return "NO_MATCH";
}

